/*

Copyright 2011 Danilo Ruffolo <danilo.ruffolo@gmail.com>

This file is part of nTetris.

nTetris is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

nTetris is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with nTetris.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include <ncurses.h>
#include <fstream>
#include <cstdlib>

#include "interfaccia.h"

Interfaccia::Interfaccia()
{
    menuPrinc[0] = "NUOVA PARTITA";
    menuPrinc[1] = "CARICA PARTITA";
    menuPrinc[2] = "VISUALIZZA CLASSIFICA";
    menuPrinc[3] = "ESCI";

    menuPau[0] = "RIPRENDI PARTITA";
    menuPau[1] = "SALVA PARTITA";
    menuPau[2] = "NUOVA PARTITA";
    menuPau[3] = "CARICA PARTITA";
    menuPau[4] = "TORNA AL MENU PRINCIPALE";
}

void Interfaccia::disegnaSchermo(const Schermo& schermo)
{
    for(int i = 4; i < 25; i++)
        for(int j = 0; j < 12; j++)
            if(schermo.getCasella(i, j) == 8)
                mvaddch(i, j+COLS/2-11, '#');

    for(int i = 0; i < 7; i++)
        for(int j = 0; j < 8; j++)
        {
            if(i > 0 && i < 6)
            {
                mvaddch(i+4, COLS/2+5, '#');
                mvaddch(i+4, COLS/2+12, '#');
            }
            else
                mvaddch(i+4, j+COLS/2+5, '#');
        }

    mvprintw(11, COLS/2+5, "Prossimo");
    mvprintw(12, COLS/2+6, "pezzo");
    mvprintw(15, COLS/2+4, "LIVELLO");
    mvprintw(18, COLS/2+4, "LINEE");
    mvprintw(21, COLS/2+4, "PUNTEGGIO");
    refresh();
}

void Interfaccia::refreshPezzi(const Schermo& schermo)
{
    for (int i = 4; i < 24; i++)
        for(int j = 1; j < 11; j++)
            if(schermo.getCasella(i, j) > 0)
            {
                attron(COLOR_PAIR(schermo.getCasella(i, j)));
                mvaddch(i, j+COLS/2-11, '@');
                attroff(COLOR_PAIR(schermo.getCasella(i, j)));
            }
            else
                mvaddch(i, j+COLS/2-11, ' ');
    refresh();
}

void Interfaccia::refreshVariabili(const Schermo& schermo)
{
    mvprintw(19, COLS/2+4, "%d", schermo.getLineeEliminate());
    mvprintw(22, COLS/2+4, "%d", schermo.getPunteggio());
    int livello = schermo.getLivello();
    
    if(livello <= 3)
        mvprintw(16, COLS/2+4, "%d", livello);
    else if(livello > 3 && livello <= 6)
    {
        attron(COLOR_PAIR(4));
        mvprintw(16, COLS/2+4, "%d", livello);
        attroff(COLOR_PAIR(4));
    }
    else if(livello > 6 && livello <= 9)
    {
        attron(COLOR_PAIR(5));
        mvprintw(16, COLS/2+4, "%d", livello);
        attroff(COLOR_PAIR(5));
    }
    else
    {
        attron(COLOR_PAIR(3));
        mvprintw(16, COLS/2+4, "%d", livello);
        attroff(COLOR_PAIR(4));
    }
    refresh();
}

void Interfaccia::stampaTETRIS()
{
    mvprintw(1, COLS/2-15, "     _______   _        _     ");
    mvprintw(2, COLS/2-15, "    |__   __| | |      (_)    ");
    mvprintw(3, COLS/2-15, "  _ __ | | ___| |_ _ __ _ ___ ");
    mvprintw(4, COLS/2-15, " | '_ \\| |/ _ \\ __| '__| / __|");
    mvprintw(5, COLS/2-15, " | | | | |  __/ |_| |  | \\__ \\");
    mvprintw(6, COLS/2-15, " |_| |_|_|\\___|\\__|_|  |_|___/");   

    mvprintw(16, COLS/2-25, "Usa FRECCIA DESTRA e FRECCIA SINISTRA per muovere i tetramini.");
    mvprintw(17, COLS/2-25, "Usa FRECCIA SU per ruotare i tetramini.");
    mvprintw(18, COLS/2-25, "Usa FRECCIA GIU per accelerare la caduta!");
    mvprintw(19, COLS/2-25, "SPAZIO o INVIO per mettere il gioco in pausa.");
    refresh();
}

void Interfaccia::stampaMenuPrincipale(int rigaEvidenziata)
{
    for(int i = 0; i < 4; i++)
    {
        if(i == rigaEvidenziata)
        {
            attron(A_REVERSE);
            mvprintw(i+10, COLS/2-10, menuPrinc[i].c_str());
            attroff(A_REVERSE);
        }
        else
            mvprintw(i+10, COLS/2-10, menuPrinc[i].c_str());
    }
    refresh();
}

void Interfaccia::stampaMenuPausa(int rigaEvidenziata)
{
    for(int i = 0; i < 5; i++)
    {
        if(i == rigaEvidenziata)
        {
            attron(A_REVERSE);
            mvprintw(i+10, COLS/2-10, menuPau[i].c_str());
            attroff(A_REVERSE);
        }
        else
            mvprintw(i+10, COLS/2-10, menuPau[i].c_str());
    }
    refresh();
}

int Interfaccia::gestioneMenuPrincipale()
{
    int voceMenu = 0;
    stampaMenuPrincipale(voceMenu);
    
    while(true)
    {
        comando = getch();
        switch(comando)
        {
        case KEY_UP:
            if(voceMenu - 1 >= 0)
            {
                voceMenu--;
                clear();
                stampaMenuPrincipale(voceMenu);
                stampaTETRIS();
            }
            break;
        case KEY_DOWN:
            if(voceMenu + 1 <= 3)
            {
                voceMenu++;
                clear();
                stampaMenuPrincipale(voceMenu);
                stampaTETRIS();
            }
            break;
        case '\n':
            clear();
            return voceMenu;
        }
    }
    return -1;
}

int Interfaccia::gestioneMenuPausa()
{
    int voceMenu = 0;
    stampaMenuPausa(voceMenu);
    
    while(true)
    {
        comando = getch();
        switch(comando)
        {
        case KEY_UP:
            if(voceMenu - 1 >= 0)
            {
                voceMenu--;
                clear();
                stampaMenuPausa(voceMenu);
                mvprintw(8, COLS/2-4, "P A U S A");
            }
            break;
        case KEY_DOWN:
            if(voceMenu + 1 <= 4)
            {
                voceMenu++;
                clear();
                stampaMenuPausa(voceMenu);
                mvprintw(8, COLS/2-4, "P A U S A");
            }
                break;
        case '\n':
            clear();
            return voceMenu;
        }
    }
    return -1;
}

int Interfaccia::menuPrincipale(Tabellone& tabellone, Schermo& schermo)
{
    stampaTETRIS();
    switch(gestioneMenuPrincipale())
    {
    case 0:
        clear();
        return 0;
    case 1:
        clear();
        carica(schermo);
        return 0;
    case 2:
        clear();
        tabellone.stampaTabellone();
        nodelay(stdscr, false);
        getch();
        nodelay(stdscr, true);
        clear();
        return 2;
    case 3:
        terminaProgramma(tabellone);
        return 1;
    }
    return 0;
}

int Interfaccia::menuPausa(Schermo& schermo, Pezzo* p, Pezzo* prossimoP, int riga, int colonna)
{
    nodelay(stdscr, false);
    clear();
    mvprintw(8, COLS/2-4, "P A U S A");
    switch(gestioneMenuPausa())
    {
    case 0:
        break;
    case 1:
        salva(schermo);
        break;
    case 2:
        clear();
        disegnaSchermo(schermo);
        schermo.nuovaPartita();
        nodelay(stdscr, true);
        return 2;
    case 3:
        carica(schermo);
        disegnaSchermo(schermo);
        nodelay(stdscr, true);
        return 2;
    case 4:
        return 3;
    }
    
    clear();
    disegnaSchermo(schermo);
    refreshPezzi(schermo);
    refreshVariabili(schermo);
    p->stampaPezzo(riga, colonna);
    prossimoP->stampaPezzo(5, 18);
    nodelay(stdscr, true);
    return 0;
}

void Interfaccia::salva(const Schermo& schermo)
{
    fstream save;
    save.open("save.dat");
    for(int i = 4; i < 24; i++)
    {
        for(int j = 1; j < 11; j++)
            save << schermo.getCasella(i, j);
        save << endl;
    }
    save << schermo.getLivello() << endl;
    save << schermo.getLineeEliminate() << endl;
    save << schermo.getPunteggio() << endl;
    save.close();
}

void Interfaccia::carica(Schermo& schermo)
{
    fstream load;
    char c;
    char* cPtr;
    clear();
    disegnaSchermo(schermo);
    load.open("save.dat");
    for(int i = 4; i < 24; i++)
    {
        for(int j = 1; j < 12; j++)
        {
            c = load.get();
            if(c != '\n')
            {
                cPtr = &c;
                schermo.setCasella(i, j, atoi(cPtr));
                refreshPezzi(schermo);
            }
        }
    }
    load >> schermo.livello;
    load >> schermo.lineeEliminate;
    load >> schermo.punteggio;
}

void Interfaccia::terminaProgramma(Tabellone& tabellone)
{
    nodelay(stdscr, false);
    clear();
    tabellone.salvaSuFile();
    mvprintw(10, COLS/2-2, "Ciao!");
    flushinp();
    getch();
    endwin();
}

void Interfaccia::fineGioco(Schermo& schermo, Tabellone& tabellone)
{
    for(int i = 4; i < 24; i++)
    {
        for (int j = 1; j < 11; j++)
            mvaddch(i, j+1+COLS/2-12, 'X');
        usleep(50000);
        refresh();
    }
    usleep(2000000);
    flushinp();
    clear();
    mvprintw(8, COLS/2-5, "Hai perso!");
    refresh();
    flushinp();
    nodelay(stdscr, false);
    getch();
    tabellone.proceduraHiScore(schermo.getLineeEliminate(), schermo.getPunteggio());
    clear();
}
