/*

Copyright 2011 Danilo Ruffolo <danilo.ruffolo@gmail.com>

This file is part of nTetris.

nTetris is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

nTetris is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with nTetris.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include <fstream>
#include <cstdlib>

#include "pezzoT.h"
#include "pezzoI.h"
#include "pezzoJ.h"
#include "pezzoS.h"
#include "pezzoO.h"
#include "pezzoZ.h"
#include "pezzoL.h"
#include "schermo.h"

Schermo::Schermo()
    : lineeEliminate(0), livello(1), punteggio(0), contaLinee(0)
{
    for(int i = 0; i < 25; i++)
        for(int j = 0; j < 12; j++)
            caselleOccupate[i][j] = 0;

    for(int i = 1; i < 25; i++)
    {
        caselleOccupate[i][0] = 8;
        caselleOccupate[i][11] = 8;
    }
    for (int j = 1; j < 11; j++)
        caselleOccupate[24][j] = 8;
}

int Schermo::getCasella(int r, int c) const
{
    return caselleOccupate[r][c];
}

int Schermo::getLivello() const
{
    return livello;
}

int Schermo::getLineeEliminate() const
{
    return lineeEliminate;
}

unsigned long Schermo::getPunteggio() const
{
    return punteggio;
}

void Schermo::setCasella(int r, int c, int valore)
{
    caselleOccupate[r][c] = valore;
}

void Schermo::setLivello(int valore)
{
    livello = valore;
}

void Schermo::setLineeEliminate(int valore)
{
    lineeEliminate = valore;
}

void Schermo::setPunteggio(unsigned long valore)
{
    punteggio = valore;
    
}

Pezzo* Schermo::generaPezzo(int pezzo, Pezzo& j, Pezzo& t, Pezzo& s, Pezzo& i, Pezzo& o, Pezzo& z, Pezzo& l)
{
    Pezzo* p;

    switch(pezzo)
    {
    case 1:
        p = &j;
        break;
    case 2:
        p = &t;
        break;
    case 3:
        p = &s;
        break;
    case 4:
        p = &i;
        break;
    case 5:
        p = &o;
        break;
    case 6:
        p = &z;
        break;
    case 7:
        p = &l;
        break;
    }
    return p;
}
          
bool Schermo::contatto(const Forma& forma, int riga, int colonna) const
{
    for(int i = 0; i < 4; i++)
        for(int j = 0; j < 4; j++)
            if(forma.getPosizione(i, j) > 0 && getCasella(riga+i, colonna+j) > 0)
                return true;
    return false;
}

void Schermo::scolpisciPezzo(const Forma& forma, int riga, int colonna)
{
    for(int i = 0; i < 4; i++)
        for(int j = 0; j < 4; j++)
        {
            if(forma.getPosizione(i, j) > 0)
                caselleOccupate[i+riga][j+colonna] = forma.getPosizione(i, j);
        }
}

void Schermo::gestioneLinee()
{
    bool linea = true;
    for(int i = 4; i < 24; i++)
    {
        linea = true;
        for(int j = 1; j < 11; j++)
            if(caselleOccupate[i][j] == 0)
               linea = false;
        if(linea)
        {
            for(int k = 1; k < 11; k++)
            {
                attron(COLOR_PAIR(caselleOccupate[i][k]));
                mvaddch(i, k+1+COLS/2-12, '*');
                attroff(COLOR_PAIR(caselleOccupate[i][k]));
            }
            setLineeEliminate(getLineeEliminate()+1);
            mvprintw(19, COLS/2+4, "%d", lineeEliminate);
            refresh();
            usleep(1000000);
            for(int l = i-1; l > 0; l--)
                for(int m = 1; m < 11; m++)
                    caselleOccupate[l+1][m] = caselleOccupate[l][m];
            for(int l = 4; l < 24; l++)
                for(int m = 1; m < 11; m++)
                {
                    attron(COLOR_PAIR(caselleOccupate[l][m]));
                    if(caselleOccupate[l][m] > 0)
                        mvaddch(l, m+1+COLS/2-12, '@');
                    else
                        mvaddch(l, m+1+COLS/2-12, ' ');
                    attroff(COLOR_PAIR(caselleOccupate[l][m]));
                }
            refresh();
            contaLinee++;
            gestioneLinee();
        }
    }
    switch(contaLinee)
    {
    case 1:
        setPunteggio(getPunteggio()+100);
        break;
    case 2:
        setPunteggio(getPunteggio()+300);
        break;
    case 3:
        setPunteggio(getPunteggio()+600);
        break;
    case 4:
        setPunteggio(getPunteggio()+1000);
        break;
    default:
        break;
        }
    refresh();
    contaLinee = 0;
    livello = lineeEliminate/10+1;
    flushinp();
}

void Schermo::muoviSx(Pezzo* p, int& riga, int& colonna)
{
    if(contatto(p->getForma(p->getFormaAttuale()), riga, colonna-1))
       return;
    p->cancellaPezzo(riga, colonna);
    colonna--;
    p->stampaPezzo(riga, colonna);
    refresh();
}

void Schermo::muoviDx(Pezzo* p, int& riga, int& colonna)
{
    if(contatto(p->getForma(p->getFormaAttuale()), riga, colonna+1))
        return;
    p->cancellaPezzo(riga, colonna);
    colonna++;
    p->stampaPezzo(riga, colonna);
    refresh();
}

void Schermo::muoviGiu(Pezzo* p, int& riga, int& colonna)
{
    if(contatto(p->getForma(p->getFormaAttuale()), riga+1, colonna))
        return;
    p->cancellaPezzo(riga, colonna);
    riga++;
    p->stampaPezzo(riga, colonna);
    refresh();
}
    
void Schermo::ruotaPezzo(Pezzo* p, int riga, int colonna)
{
    bool resettata = false;
    p->cancellaPezzo(riga, colonna);

    if(p->getFormaAttuale() + 1 > p->getMaxForme())
    {
        p->setFormaAttuale(0);
        resettata = true;
    }
    else
        p->setFormaAttuale(p->getFormaAttuale() + 1);
    
    if(contatto(p->getForma(p->getFormaAttuale()), riga, colonna))
    {
        if(resettata)
            p->setFormaAttuale(p->getMaxForme());
        else
            p->setFormaAttuale(p->getFormaAttuale() - 1);
    }
    
    p->stampaPezzo(riga, colonna);
    refresh();
}

void Schermo::nuovaPartita()
{
    lineeEliminate = 0;
    punteggio = 0;
    livello = 1;
    for (int i = 4; i < 24; i++)
        for(int j = 1; j < 11; j++)
        {
            setCasella(i, j, 0);
            mvaddch(i, j+1+COLS/2-12, ' ');
        }   
}
