/*

Copyright 2011 Danilo Ruffolo <danilo.ruffolo@gmail.com>

This file is part of nTetris.

nTetris is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

nTetris is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with nTetris.  If not, see
<http://www.gnu.org/licenses/>.

*/

#ifndef SCHERMO_H
#define SCHERMO_H

#include <ncurses.h>
#include <unistd.h>
#include <string>

#include "pezzo.h"
#include "tabellone.h"

using namespace std;

class Interfaccia;

class Schermo
{

    friend class Interfaccia;
    
 public:
    Schermo();
    
    int getCasella(int r, int c) const;
    int getLivello() const;
    int getLineeEliminate() const;
    unsigned long getPunteggio() const;
    
    void setCasella(int r, int c, int valore);
    void setLivello(int valore);
    void setLineeEliminate(int valore);
    void setPunteggio(unsigned long valore);
					
    Pezzo* generaPezzo(int pezzo, Pezzo& j, Pezzo& t, Pezzo& s, Pezzo& i, Pezzo& o, Pezzo& z, Pezzo& l);
    
    bool contatto(const Forma& forma, int riga, int colonna) const;
    void scolpisciPezzo(const Forma& forma, int riga, int colonna);

    void gestioneLinee();

    void muoviSx(Pezzo* p, int& riga, int& colonna);
    void muoviDx(Pezzo* p, int& riga, int& colonna);
    void muoviGiu(Pezzo* p, int& riga, int& colonna);
    void ruotaPezzo(Pezzo* p, int riga, int colonna);

    void nuovaPartita();

 private:
    int caselleOccupate[25][12];
    
    int lineeEliminate;
    int livello;
    unsigned long punteggio;
    int comando;
    int contaLinee;
};

#endif
