/*

Copyright 2011 Danilo Ruffolo <danilo.ruffolo@gmail.com>

This file is part of nTetris.

nTetris is free software: you can redistribute it and/or
modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

nTetris is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with nTetris.  If not, see
<http://www.gnu.org/licenses/>.

*/

#include <fstream>
#include <sstream>
#include <ncurses.h>
#include <cstdlib>


#include "tabellone.h"

Tabellone::Tabellone()
{
    fstream file;
    file.open("hiscores.dat");

    for(int i = 0; i < 10; i++)
    {
        Hiscore tmp;

        file >> tmp.nome;
        file >> tmp.linee;
        file >> tmp.punti;

        tabellone.push_back(tmp);
    }

    file.close();
}

list<Hiscore> Tabellone::getTabellone()
{
    return tabellone;
}

int Tabellone::getDistanzaDaiNomi()
{
    list<Hiscore>::iterator it;
    int distanza = 0;
    
    for(it = tabellone.begin(); it != tabellone.end(); it++)
    {
        if (it->nome.size() > distanza)
            distanza = it->nome.size();
    }
    
    distanza += 2;
    return distanza;
}

void Tabellone::stampaTabellone()
{
    list<Hiscore>::iterator it;
    int count = 0;
    clear();
    mvprintw(8, COLS/2-10, "C L A S S I F I C A");
    mvprintw(10, COLS/2-10, "NOME");
    mvprintw(10, COLS/2-12 + getDistanzaDaiNomi(), "LINEE");
    mvprintw(10, COLS/2-5 + getDistanzaDaiNomi(), "PUNTI");
    for(int i = 1; i < getDistanzaDaiNomi()+ 11; i++)
    	mvprintw(11, COLS/2-11 + i, "-");
    for(it = tabellone.begin(); it != tabellone.end(); it++)
    {
        mvprintw(count+12, COLS/2-10, it->getNome().c_str());
        mvprintw(count+12, getDistanzaDaiNomi() + COLS/2-10, it->getLinee().c_str());
        mvprintw(count+12, getDistanzaDaiNomi() + 5 + COLS/2-10, it->getPunti().c_str());
        count++;
    }
    mvprintw(count+13, COLS/2-25, "Premi un tasto per ritornare al menu principale.");
    refresh();
}

void Tabellone::proceduraHiScore(int linee, int punti)
{
    list<Hiscore>::iterator it;
    for(it = tabellone.begin(); it != tabellone.end(); it++)
    {
        if(punti > atoi(it->punti.c_str()))
        {
            clear();
            char str[80];
            mvprintw(10, COLS/2-7, "Nuovo record!");
            mvprintw(11, COLS/2-26, "Inserisci il tuo nome (niente spazi per il momento!)");
            nodelay(stdscr, false);
            echo();
            curs_set(1);
            mvgetstr(12, COLS/2-26, str);
            ostringstream os1, os2;
            os1 << linee;
            string lineeStr = os1.str();
            os2 << punti;
            string puntiStr = os2.str();
            Hiscore tmp(str, lineeStr, puntiStr);
            tabellone.insert(it, tmp);
            tabellone.pop_back();
            stampaTabellone();
            nodelay(stdscr, true);
            noecho();
            curs_set(0);
            getch();
            return;
        }
    }
}

void Tabellone::salvaSuFile()
{
    ofstream file;
    file.open("hiscores.dat");
    list<Hiscore>::iterator it;
    for(it = tabellone.begin(); it != tabellone.end(); it++)
    {
        file << it->getNome() << " " << it->getLinee() << " " << it->getPunti() << endl;
    }
    file.close();
}
